@@ -254,24 +254,24 @@ GLObject::print(ByteStream & str, int compact, int indent, int * cur_pos) const
   if (!cur_pos) { cur_pos = &local_cur_pos; }
   
   GUTF8String buffer;
-  const char * to_print=0;
   switch(type)
   {
   case NUMBER:
-    to_print=buffer.format("%d",number);
+    buffer.format("%d",number);
     break;
   case STRING:
-    to_print=make_c_string(string);
+    buffer = make_c_string(string);
     break;
   case SYMBOL:
-    to_print=buffer.format("%s",(const char *)symbol);
+    buffer.format("%s",(const char *)symbol);
     break;
   case LIST:
-    to_print=buffer.format("(%s",(const char *)name);
+    buffer.format("(%s",(const char *)name);
     break;
   case INVALID:
     break;
   }
+  const char * to_print = (const char*)buffer;
   if (!compact && *cur_pos+strlen(to_print)>70)
   {
     char ch='\n';
@@ -404,7 +404,7 @@ GLObject::get_list(void)
 void
 GLParser::skip_white_space(const char * & start)
 {
-   while(*start && isspace(*start)) start++;
+  while(*start && isspace((unsigned char)(*start))) start++;
    if (!*start) 
        G_THROW( ByteStream::EndOfFile );
 }
@@ -424,12 +424,7 @@ GLParser::get_token(const char * & start)
        start++;
        return GLToken(GLToken::CLOSE_PAR, 0);
      }
-   else if (c=='-' || (c>='0' && c<='9'))
-     {
-       return GLToken(GLToken::OBJECT,
-                      new GLObject(strtol(start, (char **) &start, 10)));
-     }
-   else if (c=='"')
+   else if (c == '"')
      {
        GUTF8String str;
        start++;
@@ -491,23 +486,26 @@ GLParser::get_token(const char * & start)
                G_THROW( ByteStream::EndOfFile );
              }
          }
-       return GLToken(GLToken::OBJECT, 
-                      new GLObject(GLObject::STRING, str));
+       return GLToken(GLToken::OBJECT, new GLObject(GLObject::STRING, str));
      }
-   else
+   else if (c=='-' || (c>='0' && c<='9'))
      {
-       GUTF8String str;
-       while(1)
-	 {
-           char ch=*start++;
-           if (!ch)
-             G_THROW( ByteStream::EndOfFile );
-           if (ch==')') { start--; break; }
-           if (isspace(ch)) break;
-           str+=ch;
-	 }
-       return GLToken(GLToken::OBJECT, new GLObject(GLObject::SYMBOL, str));
+       const char *here = start;
+       long val = strtol(start, (char**) &start, 10);
+       if (start > here)
+         return GLToken(GLToken::OBJECT, new GLObject(val));
      }
+   
+   GUTF8String str;
+   while(c != 0 && c != ')' && c != '(' && c != '"' && !isspace((unsigned char)c))
+     {
+       str += c;
+       c = *++start;
+     }
+   if (c == 0)
+     G_THROW(ByteStream::EndOfFile);
+   else
+     return GLToken(GLToken::OBJECT, new GLObject(GLObject::SYMBOL, str));
 } 
 
 void
@@ -522,7 +520,7 @@ GLParser::parse(const char * cur_name, GPList<GLObject> & list,
     GLToken token=get_token(start);
     if (token.type==GLToken::OPEN_PAR)
     {
-      if (isspace(*start))
+      if (isspace((unsigned char)(*start)))
       {
         GUTF8String mesg=GUTF8String( ERR_MSG("DjVuAnno.paren") "\t")+cur_name;
         G_THROW(mesg);
@@ -1368,13 +1366,13 @@ DjVuANT::encode_raw(void) const
        GUTF8String mdatabuffer("(");
        mdatabuffer +=  METADATA_TAG ;
        for (GPosition pos=metadata; pos; ++pos)
-         mdatabuffer +=" (" + metadata.key(pos) + make_c_string(metadata[pos]) + ")";
+         mdatabuffer +=" (" + metadata.key(pos) + " " + make_c_string(metadata[pos]) + ")";
        mdatabuffer += " )";
        parser.parse(mdatabuffer);
      }
       //*** XMP Metadata
    del_all_items(XMP_TAG, parser);
-   if (!xmpmetadata)
+   if (!!xmpmetadata)
      {
        GUTF8String mdatabuffer("(");
        mdatabuffer +=  XMP_TAG;
@@ -1384,8 +1382,10 @@ DjVuANT::encode_raw(void) const
      //*** Mapareas
    del_all_items(GMapArea::MAPAREA_TAG, parser);
    for(GPosition pos=map_areas;pos;++pos)
-      parser.parse(map_areas[pos]->print());
-
+     {
+       GUTF8String mapareabuffer = map_areas[pos]->print();
+       parser.parse(mapareabuffer);
+     }
    GP<ByteStream> gstr=ByteStream::create();
    ByteStream &str=*gstr;
    parser.print(str, 1);
@@ -1401,7 +1401,7 @@ DjVuANT::is_empty(void) const
 {
    GUTF8String raw=encode_raw();
    for(int i=raw.length()-1;i>=0;i--)
-      if (isspace(raw[i])) raw.setat(i, 0);
+     if (isspace((unsigned char)raw[i])) raw.setat(i, 0);
       else break;
    return raw.length()==0;
 }
