@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: GString.cpp,v 1.27 2009/05/17 23:57:42 leonb Exp $
-// $Name: release_3_5_22 $
 
 // From: Leon Bottou, 1/31/2002
 // This file has very little to do with my initial implementation.
@@ -76,6 +73,7 @@
 #include "GThreads.h"
 #include "debug.h"
 
+#include <stddef.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -90,16 +88,12 @@
 #endif
 #include <ctype.h>
 
+#ifndef LC_NUMERIC          //MingW
+# undef DO_CHANGELOCALE
+# define LC_NUMERIC 0
+#endif
 #ifndef DO_CHANGELOCALE
-#define DO_CHANGELOCALE 1
-#ifdef UNIX
-#if THREADMODEL != COTHREADS
-#if THREADMODEL != NOTHREADS
-#undef DO_CHANGELOCALE
-#define DO_CHANGELOCALE 0
-#endif
-#endif 
-#endif
+# define DO_CHANGELOCALE 0
 #endif
 
 
@@ -279,7 +273,9 @@ public:
   ~ChangeLocale();
 private:
   GUTF8String locale;
+#if DO_CHANGELOCALE
   int category;
+#endif
 };
 
 class GStringRep::Native : public GStringRep
@@ -458,7 +454,9 @@ GStringRep::Native::ncopy(
 }
 
 GStringRep::ChangeLocale::ChangeLocale(const int xcategory, const char xlocale[] )
+#if DO_CHANGELOCALE
   : category(xcategory)
+#endif
 {
 #if DO_CHANGELOCALE
   // This is disabled under UNIX because 
@@ -542,25 +540,23 @@ GStringRep::Native::UCS4toString(
 // the results with a nill termination.  (Normally 7 characters
 // is enough.)
 unsigned char *
-GStringRep::UCS4toNative(
-  const uint32_t w0,unsigned char *ptr, mbstate_t *ps)
+GStringRep::UCS4toNative(const uint32_t w0,unsigned char *ptr, mbstate_t *ps)
 {
   uint16_t w1;
   uint16_t w2=1;
-  for(int count=(sizeof(wchar_t)==sizeof(w1)) ? UCS4toUTF16(w0,w1,w2) : 1;
+  for(int count=(sizeof(wchar_t)==sizeof(w1)) 
+        ? UCS4toUTF16(w0,w1,w2) : 1;
       count;
       --count,w1=w2)
-  {
-    // wchar_t can be either UCS4 or UCS2
-    const wchar_t w=(sizeof(wchar_t) == sizeof(w1))?(wchar_t)w1:(wchar_t)w0;
-    int i=wcrtomb((char *)ptr,w,ps);
-    if(i<0)
     {
-      break;
+      // wchar_t can be either UCS4 or UCS2
+      const wchar_t w=(sizeof(wchar_t) == sizeof(w1))?(wchar_t)w1:(wchar_t)w0;
+      int i=wcrtomb((char *)ptr,w,ps);
+      if(i<0)
+        break;
+      ptr[i]=0;
+      ptr += i;
     }
-    ptr[i]=0;
-    ptr += i;
-  }
   ptr[0]=0;
   return ptr;
 }
@@ -645,26 +641,22 @@ GBaseString::UTF8ToNative(
   const char *source=(*this);
   GP<GStringRep> retval;
   if(source && source[0]) 
-  {
-#if DO_CHANGELOCALE
-    GUTF8String lc_ctype(setlocale(LC_CTYPE,0));
-#endif
-    bool repeat;
-    for(repeat=!currentlocale;;repeat=false)
     {
-      retval=(*this)->toNative((GStringRep::EscapeMode)escape);
 #if DO_CHANGELOCALE
-      if (!repeat || retval || (lc_ctype == setlocale(LC_CTYPE,"")))
+      GUTF8String lc_ctype(setlocale(LC_CTYPE,0));
+      bool repeat;
+      for(repeat=!currentlocale;;repeat=false)
+        {
 #endif
-        break;
-    }
+          retval=(*this)->toNative((GStringRep::EscapeMode)escape);
 #if DO_CHANGELOCALE
-    if(!repeat)
-      {
+          if (!repeat || retval || (lc_ctype == setlocale(LC_CTYPE,"")))
+            break;
+        }
+      if(!repeat)
         setlocale(LC_CTYPE,(const char *)lc_ctype);
-      }
 #endif
-  }
+    }
   return GNativeString(retval);
 }
 
@@ -699,27 +691,19 @@ GBaseString::NativeToUTF8(void) const
     const char *source=(*this);
 #if DO_CHANGELOCALE
     GUTF8String lc_ctype=setlocale(LC_CTYPE,0);
-#endif
     bool repeat;
     for(repeat=true;;repeat=false)
-    {
-      if( (retval=GStringRep::NativeToUTF8(source)) )
       {
-        if(GStringRep::cmp(retval->toNative(),source))
-        {
-          retval=GStringRep::UTF8::create((unsigned int)0);
-        }
-      }
-#if DO_CHANGELOCALE
-      if(!repeat || retval || (lc_ctype == setlocale(LC_CTYPE,"")))
 #endif
-        break;
-    }
+        if( (retval=GStringRep::NativeToUTF8(source)) )
+          if(GStringRep::cmp(retval->toNative(),source))
+            retval=GStringRep::UTF8::create((unsigned int)0);
 #if DO_CHANGELOCALE
+        if(!repeat || retval || (lc_ctype == setlocale(LC_CTYPE,"")))
+          break;
+      }
     if(!repeat)
-    {
       setlocale(LC_CTYPE,(const char *)lc_ctype);
-    }
 #endif
   }
   return GUTF8String(retval);
@@ -1232,11 +1216,11 @@ GP<GStringRep>
 GStringRep::getbuf(int n) const
 {
   GP<GStringRep> retval;
-  if(n< 0)
+  if(n < 0)
     n=strlen(data);
-  if(n>0)
+  if(n >= 0)
   {
-    retval=blank(n);
+    retval=blank((n>0) ? n : 1);
     char *ndata=retval->data;
     strncpy(ndata,data,n);
     ndata[n]=0;
@@ -1245,17 +1229,19 @@ GStringRep::getbuf(int n) const
 }
 
 const char *
-GStringRep::isCharType(
-  bool (*xiswtest)(const unsigned long wc), const char *ptr, const bool reverse) const
+GStringRep::isCharType(bool (*xiswtest)(const unsigned long wc), 
+                       const char *ptr, 
+                       const bool reverse) const
 {
-  char const * xptr=ptr;
-  const unsigned long w=getValidUCS4(xptr);
-  if((ptr != xptr)
-    &&(((sizeof(wchar_t) == 2)&&(w&~0xffff))
-      ||(reverse?(!xiswtest(w)):xiswtest(w))))
-  {
-    ptr=xptr;
-  }
+  const char *xptr = ptr;
+  unsigned long w=getValidUCS4(xptr);
+  if(ptr != xptr)
+    {
+      if (sizeof(wchar_t) == 2)
+        w &= 0xffff;
+      if (reverse ^ xiswtest(w))
+        ptr = xptr;
+    }
   return ptr;
 }
 
@@ -1295,9 +1281,9 @@ bool
 GStringRep::giswspace(const unsigned long w)
 {
 #if HAS_WCTYPE
-  return iswspace((wchar_t)w);
+  return !!iswspace((wchar_t)w);
 #else
-  if (w & ~0xff) || isspace((int)(w & 0xff));
+  return (w & ~0xff) ? false : !!isspace((int)(w & 0xff));
 #endif
 }
 
@@ -1305,9 +1291,9 @@ bool
 GStringRep::giswupper(const unsigned long w)
 {
 #if HAS_WCTYPE
-  return iswupper((wchar_t)w);
+  return !!iswupper((wchar_t)w);
 #else
-  return (w & ~0xff) || isupper((int)(w & 0xff));
+  return (w & ~0xff) ? false : !!isupper((int)(w & 0xff));
 #endif
 }
 
@@ -1315,9 +1301,9 @@ bool
 GStringRep::giswlower(const unsigned long w)
 {
 #if HAS_WCTYPE
-  return iswlower((wchar_t)w);
+  return !!iswlower((wchar_t)w);
 #else
-  return (w & ~0xff) || islower((int)(w & 0xff));
+  return (w & ~0xff) ? false : !!islower((int)(w & 0xff));
 #endif
 }
 
@@ -1412,7 +1398,8 @@ GStringRep::toEscaped( const bool tosevenbit ) const
   char const *s=start;
   char const *last=s;
   GP<GStringRep> special;
-  for(unsigned long w;(w=getValidUCS4(s));last=s)
+  for(unsigned long w;(w=getValidUCS4(s));last=s)  
+    // Whoever wrote this for statement should be __complete_here__
   {
     char const *ss=0;
     switch(w)
@@ -1597,7 +1584,7 @@ GStringRep::setat(int n, char ch) const
 
 #if defined(AUTOCONF) && defined(HAVE_VSNPRINTF)
 # define USE_VSNPRINTF vsnprintf
-#elif defined(WIN32) && !defined(__CYGWIN32__)
+#elif defined(_WIN32) && !defined(__CYGWIN32__)
 # define USE_VSNPRINTF _vsnprintf
 #elif defined(linux)
 # define USE_VSNPRINTF vsnprintf
@@ -1812,41 +1799,43 @@ GStringRep::UTF8::UCS4toString(
 }
 
 int
-GStringRep::UTF8::ncopy(
-  wchar_t * const buf, const int buflen ) const
+GStringRep::UTF8::ncopy(wchar_t * const buf, const int buflen ) const
 {
   int retval=(-1);
   if(buf && buflen)
-  {
-	buf[0]=0;
-    if(data[0])
+    {
+      buf[0]=0;
+      if(data[0])
 	{
-      const size_t length=strlen(data);
-      const unsigned char * const eptr=(const unsigned char *)(data+length);
+          const size_t length=strlen(data);
+          const unsigned char * const eptr=(const unsigned char *)(data+length);
 	  wchar_t *r=buf;
 	  wchar_t const * const rend=buf+buflen;
-      for(const unsigned char *s=(const unsigned char *)data;(r<rend)&&(s<eptr)&&*s;)
-	  {
-        const uint32_t w0=UTF8toUCS4(s,eptr);
-        uint16_t w1;
-        uint16_t w2=1;
-        for(int count=(sizeof(wchar_t) == sizeof(w1))?UCS4toUTF16(w0,w1,w2):1;
-            count&&(r<rend);
-            --count,w1=w2,++r)
+          for(const unsigned char *s=(const unsigned char *)data;
+              (r<rend)&&(s<eptr)&&*s;)
+            {
+              const uint32_t w0=UTF8toUCS4(s,eptr);
+              uint16_t w1;
+              uint16_t w2=1;
+              for(int count=(sizeof(wchar_t)==sizeof(w1))
+                    ?UCS4toUTF16(w0,w1,w2):1;
+                  count&&(r<rend);
+                  --count,w1=w2,++r)
 		{
 		  r[0]=(sizeof(wchar_t) == sizeof(w1))?(wchar_t)w1:(wchar_t)w0;
 		}
-	  }
+            }
 	  if(r<rend)
-	  {
-	    r[0]=0;
-		retval=((size_t)r-(size_t)buf)/sizeof(wchar_t);
-	  }
-	}else
+            {
+              r[0]=0;
+              retval=((size_t)r-(size_t)buf)/sizeof(wchar_t);
+            }
+	}
+      else
 	{
 	  retval=0;
 	}
-  }
+    }
   return retval;
 }
 
@@ -1864,26 +1853,36 @@ GStringRep::UTF8::toNative(const EscapeMode escape) const
     mbstate_t ps;
     memset(&ps,0,sizeof(mbstate_t));
     for(const unsigned char *s=(const unsigned char *)data;(s<eptr)&& *s;)
-    {
-      const uint32_t w0=UTF8toUCS4(s,eptr);
-      const unsigned char * const r0=r;
-      r=UCS4toNative(w0,r,&ps);
-      if(r == r0)
       {
-        if(escape == IS_ESCAPED)
-        {
-          sprintf((char *)r,"&#%lu;",(unsigned long)w0);
-          r+=strlen((char *)r);
-        }else
-        {
-          r=buf;
-          break;
-        }
+        const unsigned char * const s0 = s;
+        const uint32_t w0=UTF8toUCS4(s,eptr);
+        if (s == s0)
+          {
+            s += 1;
+            *r++ = '?';
+          }
+        else
+          {
+            const unsigned char * const r0 = r;
+            r=UCS4toNative(w0,r,&ps);
+            if(r == r0)
+              {
+                if (escape == IS_ESCAPED)
+                  {
+                    sprintf((char *)r,"&#%lu;",(unsigned long)w0);
+                    r += strlen((char *)r);
+                  }
+                else
+                  {
+                    *r++ = '?';
+                  }
+              }
+          }
       }
-    }
     r[0]=0;
     retval = NATIVE_CREATE( (const char *)buf );
-  } else
+  } 
+  else
   {
     retval = NATIVE_CREATE( (unsigned int)0 );
   }
@@ -2150,19 +2149,6 @@ GStringRep::concat(const GP<GStringRep> &s1,const GP<GStringRep> &s2) const
   return retval;
 }
 
-#ifdef WIN32
-static const char *setlocale_win32(void)
-{
-  static const char *locale=setlocale(LC_ALL,0);
-  if(! locale || (locale[0] == 'C' && !locale[1]))
-  {
-    locale=setlocale(LC_ALL,"");
-  }
-  return locale;
-}
-const char *setlocale_win32_var = setlocale_win32();
-#endif
-
 GStringRep::GStringRep(void)
 {
   size=0;
@@ -2277,8 +2263,9 @@ GStringRep::UTF8::toLong(
     endpos=edata-data;
   }else
   {
+    GP<GStringRep> ptr = GStringRep::UTF8::create();
     endpos=(-1);
-    GP<GStringRep> ptr=ptr->strdup(data+pos);
+    ptr=ptr->strdup(data+pos);
     if(ptr)
       ptr=ptr->toNative(NOT_ESCAPED);
     if(ptr)
@@ -2322,8 +2309,9 @@ GStringRep::UTF8::toULong(
     endpos=edata-data;
   }else
   {
+    GP<GStringRep> ptr = GStringRep::UTF8::create();
     endpos=(-1);
-    GP<GStringRep> ptr=ptr->strdup(data+pos);
+    ptr=ptr->strdup(data+pos);
     if(ptr)
       ptr=ptr->toNative(NOT_ESCAPED);
     if(ptr)
@@ -2366,8 +2354,9 @@ GStringRep::UTF8::toDouble(const int pos, int &endpos) const
     endpos=edata-data;
   }else
   {
+    GP<GStringRep> ptr = GStringRep::UTF8::create();
     endpos=(-1);
-    GP<GStringRep> ptr=ptr->strdup(data+pos);
+    ptr=ptr->strdup(data+pos);
     if(ptr)
       ptr=ptr->toNative(NOT_ESCAPED);
     if(ptr)
